@@ -29,6 +29,7 @@ import requests
 
 from .NotifyBase import NotifyBase
 from ..common import NotifyType
+from ..common import NotifyFormat
 from ..utils import parse_list
 from ..utils import validate_regex
 from ..AppriseLocale import gettext_lazy as _
@@ -195,6 +196,16 @@ class NotifyPushover(NotifyBase):
             'regex': (r'^[a-z]{1,12}$', 'i'),
             'default': PushoverSound.PUSHOVER,
         },
+        'url': {
+            'name': _('URL'),
+            'map_to': 'supplemental_url',
+            'type': 'string',
+        },
+        'url_title': {
+            'name': _('URL Title'),
+            'map_to': 'supplemental_url_title',
+            'type': 'string'
+        },
         'retry': {
             'name': _('Retry'),
             'type': 'int',
@@ -214,7 +225,8 @@ class NotifyPushover(NotifyBase):
     })
 
     def __init__(self, user_key, token, targets=None, priority=None,
-                 sound=None, retry=None, expire=None, **kwargs):
+                 sound=None, retry=None, expire=None, supplemental_url=None,
+                 supplemental_url_title=None, **kwargs):
         """
         Initialize Pushover Object
         """
@@ -240,6 +252,10 @@ class NotifyPushover(NotifyBase):
         if len(self.targets) == 0:
             self.targets = (PUSHOVER_SEND_TO_ALL, )
 
+        # Setup supplemental url
+        self.supplemental_url = supplemental_url
+        self.supplemental_url_title = supplemental_url_title
+
         # Setup our sound
         self.sound = NotifyPushover.default_pushover_sound \
             if not isinstance(sound, six.string_types) else sound.lower()
@@ -320,6 +336,15 @@ class NotifyPushover(NotifyBase):
                 'sound': self.sound,
             }
 
+            if self.supplemental_url:
+                payload['url'] = self.supplemental_url
+            if self.supplemental_url_title:
+                payload['url_title'] = self.supplemental_url_title
+
+            if self.notify_format == NotifyFormat.HTML:
+                # https://pushover.net/api#html
+                payload['html'] = 1
+
             if self.priority == PushoverPriority.EMERGENCY:
                 payload.update({'retry': self.retry, 'expire': self.expire})
 
@@ -564,6 +589,14 @@ class NotifyPushover(NotifyBase):
             results['sound'] = \
                 NotifyPushover.unquote(results['qsd']['sound'])
 
+        # Get the supplementary url
+        if 'url' in results['qsd'] and len(results['qsd']['url']):
+            results['supplemental_url'] = NotifyPushover.unquote(
+                results['qsd']['url']
+            )
+        if 'url_title' in results['qsd'] and len(results['qsd']['url_title']):
+            results['supplemental_url_title'] = results['qsd']['url_title']
+
         # Get expire and retry
         if 'expire' in results['qsd'] and len(results['qsd']['expire']):
             results['expire'] = results['qsd']['expire']
