@@ -43,6 +43,7 @@ from apprise import NotifyImageSize
 from apprise import __version__
 from apprise import URLBase
 from apprise import PrivacyMode
+from apprise.AppriseLocale import LazyTranslation
 
 from apprise.plugins import SCHEMA_MAP
 from apprise.plugins import __load_matrix
@@ -54,6 +55,15 @@ import inspect
 import logging
 logging.disable(logging.CRITICAL)
 
+# Sending notifications requires the coroutines to be awaited, so we need to
+# wrap the original function when mocking it. But don't import for Python 2.
+if not six.PY2:
+    import apprise.py3compat.asyncio as py3aio
+else:
+    class py3aio:
+        def notify():
+            pass
+
 # Attachment Directory
 TEST_VAR_DIR = join(dirname(__file__), 'var')
 
@@ -63,6 +73,25 @@ def test_apprise():
     API: Apprise() object
 
     """
+    def do_notify(server, *args, **kwargs):
+        return server.notify(*args, **kwargs)
+
+    apprise_test(do_notify)
+
+
+@pytest.mark.skipif(sys.version_info.major <= 2, reason="Requires Python 3.x+")
+def test_apprise_async():
+    """
+    API: Apprise() object asynchronous methods
+
+    """
+    def do_notify(server, *args, **kwargs):
+        return py3aio.tosync(server.async_notify(*args, **kwargs))
+
+    apprise_test(do_notify)
+
+
+def apprise_test(do_notify):
     # Caling load matix a second time which is an internal function causes it
     # to skip over content already loaded into our matrix and thefore accesses
     # other if/else parts of the code that aren't otherwise called
@@ -153,7 +182,7 @@ def test_apprise():
     a.clear()
 
     # No servers to notify
-    assert a.notify(title="my title", body="my body") is False
+    assert do_notify(a, title="my title", body="my body") is False
 
     class BadNotification(NotifyBase):
         def __init__(self, **kwargs):
@@ -162,7 +191,7 @@ def test_apprise():
             # We fail whenever we're initialized
             raise TypeError()
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -176,7 +205,7 @@ def test_apprise():
             super(GoodNotification, self).__init__(
                 notify_format=NotifyFormat.HTML, **kwargs)
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -202,8 +231,8 @@ def test_apprise():
     assert len(a) == 0
 
     # We'll fail because we've got nothing to notify
-    assert a.notify(
-        title="my title", body="my body") is False
+    assert do_notify(
+        a, title="my title", body="my body") is False
 
     # Clear our server listings again
     a.clear()
@@ -213,50 +242,50 @@ def test_apprise():
 
     # Bad Notification Type is still allowed as it is presumed the user
     # know's what their doing
-    assert a.notify(
-        title="my title", body="my body", notify_type='bad') is True
+    assert do_notify(
+        a, title="my title", body="my body", notify_type='bad') is True
 
     # No Title/Body combo's
-    assert a.notify(title=None, body=None) is False
-    assert a.notify(title='', body=None) is False
-    assert a.notify(title=None, body='') is False
+    assert do_notify(a, title=None, body=None) is False
+    assert do_notify(a, title='', body=None) is False
+    assert do_notify(a, title=None, body='') is False
 
     # As long as one is present, we're good
-    assert a.notify(title=None, body='present') is True
-    assert a.notify(title='present', body=None) is True
-    assert a.notify(title="present", body="present") is True
+    assert do_notify(a, title=None, body='present') is True
+    assert do_notify(a, title='present', body=None) is True
+    assert do_notify(a, title="present", body="present") is True
 
     # Send Attachment with success
     attach = join(TEST_VAR_DIR, 'apprise-test.gif')
-    assert a.notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a, body='body', title='test', notify_type=NotifyType.INFO,
         attach=attach) is True
 
     # Send the attachment as an AppriseAttachment object
-    assert a.notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a, body='body', title='test', notify_type=NotifyType.INFO,
         attach=AppriseAttachment(attach)) is True
 
     # test a invalid attachment
-    assert a.notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a, body='body', title='test', notify_type=NotifyType.INFO,
         attach='invalid://') is False
 
     # Repeat the same tests above...
     # however do it by directly accessing the object; this grants the similar
     # results:
-    assert a[0].notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a[0], body='body', title='test', notify_type=NotifyType.INFO,
         attach=attach) is True
 
     # Send the attachment as an AppriseAttachment object
-    assert a[0].notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a[0], body='body', title='test', notify_type=NotifyType.INFO,
         attach=AppriseAttachment(attach)) is True
 
     # test a invalid attachment
-    assert a[0].notify(
-        body='body', title='test', notify_type=NotifyType.INFO,
+    assert do_notify(
+        a[0], body='body', title='test', notify_type=NotifyType.INFO,
         attach='invalid://') is False
 
     class ThrowNotification(NotifyBase):
@@ -264,7 +293,7 @@ def test_apprise():
             # Pretend everything is okay
             raise TypeError()
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -273,7 +302,7 @@ def test_apprise():
             # Pretend everything is okay
             raise RuntimeError()
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -283,7 +312,7 @@ def test_apprise():
             # Pretend everything is okay
             return False
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -310,7 +339,7 @@ def test_apprise():
 
         # Test when our notify both throws an exception and or just
         # simply returns False
-        assert a.notify(title="present", body="present") is False
+        assert do_notify(a, title="present", body="present") is False
 
     # Create a Notification that throws an unexected exception
     class ThrowInstantiateNotification(NotifyBase):
@@ -318,7 +347,7 @@ def test_apprise():
             # Pretend everything is okay
             raise TypeError()
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -498,7 +527,27 @@ def test_apprise_tagging(mock_post, mock_get):
     API: Apprise() object tagging functionality
 
     """
+    def do_notify(server, *args, **kwargs):
+        return server.notify(*args, **kwargs)
 
+    apprise_tagging_test(mock_post, mock_get, do_notify)
+
+
+@mock.patch('requests.get')
+@mock.patch('requests.post')
+@pytest.mark.skipif(sys.version_info.major <= 2, reason="Requires Python 3.x+")
+def test_apprise_tagging_async(mock_post, mock_get):
+    """
+    API: Apprise() object tagging functionality asynchronous methods
+
+    """
+    def do_notify(server, *args, **kwargs):
+        return py3aio.tosync(server.async_notify(*args, **kwargs))
+
+    apprise_tagging_test(mock_post, mock_get, do_notify)
+
+
+def apprise_tagging_test(mock_post, mock_get, do_notify):
     # A request
     robj = mock.Mock()
     setattr(robj, 'raw', mock.Mock())
@@ -535,18 +584,19 @@ def test_apprise_tagging(mock_post, mock_get):
 
     # notify the awesome tag; this would notify both services behind the
     # scenes
-    assert a.notify(title="my title", body="my body", tag='awesome') is True
+    assert do_notify(
+        a, title="my title", body="my body", tag='awesome') is True
 
     # notify all of the tags
-    assert a.notify(
-        title="my title", body="my body", tag=['awesome', 'mmost']) is True
+    assert do_notify(
+        a, title="my title", body="my body", tag=['awesome', 'mmost']) is True
 
     # When we query against our loaded notifications for a tag that simply
     # isn't assigned to anything, we return None.  None (different then False)
     # tells us that we litterally had nothing to query.  We didn't fail...
     # but we also didn't do anything...
-    assert a.notify(
-        title="my title", body="my body", tag='missing') is None
+    assert do_notify(
+        a, title="my title", body="my body", tag='missing') is None
 
     # Now to test the ability to and and/or notifications
     a = Apprise()
@@ -571,20 +621,20 @@ def test_apprise_tagging(mock_post, mock_get):
     # Matches the following only:
     #   - json://localhost/tagCD/
     #   - json://localhost/tagCDE/
-    assert a.notify(
-        title="my title", body="my body", tag=[('TagC', 'TagD')]) is True
+    assert do_notify(
+        a, title="my title", body="my body", tag=[('TagC', 'TagD')]) is True
 
     # Expression: (TagY and TagZ) or TagX
     # Matches nothing, None is returned in this case
-    assert a.notify(
-        title="my title", body="my body",
+    assert do_notify(
+        a, title="my title", body="my body",
         tag=[('TagY', 'TagZ'), 'TagX']) is None
 
     # Expression: (TagY and TagZ) or TagA
     # Matches the following only:
     #   - json://localhost/tagAB/
-    assert a.notify(
-        title="my title", body="my body",
+    assert do_notify(
+        a, title="my title", body="my body",
         tag=[('TagY', 'TagZ'), 'TagA']) is True
 
     # Expression: (TagE and TagD) or TagB
@@ -592,8 +642,8 @@ def test_apprise_tagging(mock_post, mock_get):
     #   - json://localhost/tagCDE/
     #   - json://localhost/tagAB/
     #   - json://localhost/tagB/
-    assert a.notify(
-        title="my title", body="my body",
+    assert do_notify(
+        a, title="my title", body="my body",
         tag=[('TagE', 'TagD'), 'TagB']) is True
 
     # Garbage Entries in tag field just get stripped out. the below
@@ -602,8 +652,8 @@ def test_apprise_tagging(mock_post, mock_get):
     # we fail.  None is returned as a way of letting us know that we
     # had Notifications to notify, but since none of them matched our tag
     # none were notified.
-    assert a.notify(
-        title="my title", body="my body",
+    assert do_notify(
+        a, title="my title", body="my body",
         tag=[(object, ), ]) is None
 
 
@@ -695,7 +745,7 @@ def test_apprise_notify_formats(tmpdir):
             # Pretend everything is okay
             return True
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -711,7 +761,7 @@ def test_apprise_notify_formats(tmpdir):
             # Pretend everything is okay
             return True
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -727,7 +777,7 @@ def test_apprise_notify_formats(tmpdir):
             # Pretend everything is okay
             return True
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
@@ -912,6 +962,143 @@ def test_apprise_asset(tmpdir):
         'http://localhost/default/info-256x256.test'
 
 
+def test_apprise_disabled_plugins():
+    """
+    API: Apprise() Disabled Plugin States
+
+    """
+    # Reset our matrix
+    __reset_matrix()
+
+    class TestDisabled01Notification(NotifyBase):
+        """
+        This class is used to test a pre-disabled state
+        """
+
+        # Just flat out disable our service
+        enabled = False
+
+        # we'll use this as a key to make our service easier to find
+        # in the next part of the testing
+        service_name = 'na01'
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def notify(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['na01'] = TestDisabled01Notification
+
+    class TestDisabled02Notification(NotifyBase):
+        """
+        This class is used to test a post-disabled state
+        """
+
+        # we'll use this as a key to make our service easier to find
+        # in the next part of the testing
+        service_name = 'na02'
+
+        def __init__(self, *args, **kwargs):
+            super(TestDisabled02Notification, self).__init__(**kwargs)
+
+            # enable state changes **AFTER** we initialize
+            self.enabled = False
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def notify(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['na02'] = TestDisabled02Notification
+
+    # Create our Apprise instance
+    a = Apprise()
+
+    result = a.details(lang='ca-en', show_disabled=True)
+    assert isinstance(result, dict)
+    assert 'schemas' in result
+    assert len(result['schemas']) == 2
+
+    # our na01 is disabled right from the get-go
+    entry = next((x for x in result['schemas']
+                  if x['service_name'] == 'na01'), None)
+    assert entry is not None
+    assert entry['enabled'] is False
+
+    plugin = a.instantiate('na01://localhost')
+    # Object is just flat out disabled... nothing is instatiated
+    assert plugin is None
+
+    # our na02 isn't however until it's initialized; as a result
+    # it get's returned in our result set
+    entry = next((x for x in result['schemas']
+                  if x['service_name'] == 'na02'), None)
+    assert entry is not None
+    assert entry['enabled'] is True
+
+    plugin = a.instantiate('na02://localhost')
+    # Object isn't disabled until the __init__() call.  But this is still
+    # enough to not instantiate the object:
+    assert plugin is None
+
+    # If we choose to filter our disabled, we can't unfortunately filter those
+    # that go disabled after instantiation, but we do filter out any that are
+    # already known to not be enabled:
+    result = a.details(lang='ca-en', show_disabled=False)
+    assert isinstance(result, dict)
+    assert 'schemas' in result
+    assert len(result['schemas']) == 1
+
+    # We'll add a good notification to our list
+    class TesEnabled01Notification(NotifyBase):
+        """
+        This class is just a simple enabled one
+        """
+
+        # we'll use this as a key to make our service easier to find
+        # in the next part of the testing
+        service_name = 'good'
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['good'] = TesEnabled01Notification
+
+    # The last thing we'll simulate is a case where the plugin is just
+    # disabled at a later time long into it's life.  this is just to allow
+    # administrators to stop the flow of their notifications for their own
+    # given reasons.
+    plugin = a.instantiate('good://localhost')
+    assert isinstance(plugin, NotifyBase)
+
+    # we'll toggle our state
+    plugin.enabled = False
+
+    # As a result, we can now no longer send a notification:
+    assert plugin.notify("My Message") is False
+
+    # As just a proof of how you can toggle the state back:
+    plugin.enabled = True
+
+    # our notifications will go okay now
+    assert plugin.notify("My Message") is True
+
+    # Reset our matrix
+    __reset_matrix()
+    __load_matrix()
+
+
 def test_apprise_details():
     """
     API: Apprise() Details
@@ -1017,22 +1204,196 @@ def test_apprise_details():
             }
         })
 
-        def url(self):
+        def url(self, **kwargs):
             # Support URL
             return ''
 
-        def notify(self, **kwargs):
+        def send(self, **kwargs):
             # Pretend everything is okay (so we don't break other tests)
             return True
 
     # Store our good detail notification in our schema map
     SCHEMA_MAP['details'] = TestDetailNotification
 
+    # This is a made up class that is just used to verify
+    class TestReq01Notification(NotifyBase):
+        """
+        This class is used to test various requirement configurations
+        """
+
+        # Set some requirements
+        requirements = {
+            'packages_required': [
+                'cryptography <= 3.4',
+                'ultrasync',
+            ],
+            'packages_recommended': 'django',
+        }
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['req01'] = TestReq01Notification
+
+    # This is a made up class that is just used to verify
+    class TestReq02Notification(NotifyBase):
+        """
+        This class is used to test various requirement configurations
+        """
+
+        # Just not enabled at all
+        enabled = False
+
+        # Set some requirements
+        requirements = {
+            # None and/or [] is implied, but jsut to show that the code won't
+            # crash if explicitly set this way:
+            'packages_required': None,
+
+            'packages_recommended': [
+                'cryptography <= 3.4',
+            ]
+        }
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['req02'] = TestReq02Notification
+
+    # This is a made up class that is just used to verify
+    class TestReq03Notification(NotifyBase):
+        """
+        This class is used to test various requirement configurations
+        """
+
+        # Set some requirements
+        requirements = {
+            # We can over-ride the default details assigned to our plugin if
+            # specified
+            'details': _('some specified requirement details'),
+
+            # We can set a string value as well (it does not have to be a list)
+            'packages_recommended': 'cryptography <= 3.4'
+        }
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['req03'] = TestReq03Notification
+
+    # This is a made up class that is just used to verify
+    class TestReq04Notification(NotifyBase):
+        """
+        This class is used to test a case where our requirements is fixed
+        to a None
+        """
+
+        # This is the same as saying there are no requirements
+        requirements = None
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['req04'] = TestReq04Notification
+
+    # This is a made up class that is just used to verify
+    class TestReq05Notification(NotifyBase):
+        """
+        This class is used to test a case where only packages_recommended
+        is identified
+        """
+
+        requirements = {
+            # We can set a string value as well (it does not have to be a list)
+            'packages_recommended': 'cryptography <= 3.4'
+        }
+
+        def url(self, **kwargs):
+            # Support URL
+            return ''
+
+        def send(self, **kwargs):
+            # Pretend everything is okay (so we don't break other tests)
+            return True
+
+    SCHEMA_MAP['req05'] = TestReq05Notification
+
     # Create our Apprise instance
     a = Apprise()
 
     # Dictionary response
-    assert isinstance(a.details(), dict)
+    result = a.details()
+    assert isinstance(result, dict)
+
+    # Test different variations of our call
+    result = a.details(lang='ca-fr')
+    assert isinstance(result, dict)
+    for entry in result['schemas']:
+        # Verify our key does not exist because we did not ask for it
+        assert 'enabled' not in entry
+        assert 'requirements' not in entry
+
+    result = a.details(lang='us-en', show_requirements=True)
+    assert isinstance(result, dict)
+    for entry in result['schemas']:
+        # Verify our key does not exist because we did not ask for it
+        assert 'enabled' not in entry
+
+        # Requirements are set for display
+        assert 'requirements' in entry
+        assert 'details' in entry['requirements']
+        assert 'packages_required' in entry['requirements']
+        assert 'packages_recommended' in entry['requirements']
+        assert isinstance(entry['requirements']['details'], six.string_types)
+        assert isinstance(entry['requirements']['packages_required'], list)
+        assert isinstance(entry['requirements']['packages_recommended'], list)
+
+    result = a.details(lang='ca-en', show_disabled=True)
+    assert isinstance(result, dict)
+    for entry in result['schemas']:
+        # Verify that our plugin state is available to us
+        assert 'enabled' in entry
+        assert isinstance(entry['enabled'], bool)
+
+        # Verify our key does not exist because we did not ask for it
+        assert 'requirements' not in entry
+
+    result = a.details(
+        lang='ca-fr', show_requirements=True, show_disabled=True)
+    assert isinstance(result, dict)
+    for entry in result['schemas']:
+        # Plugin States are set for display
+        assert 'enabled' in entry
+        assert isinstance(entry['enabled'], bool)
+
+        # Requirements are set for display
+        assert 'requirements' in entry
+        assert 'details' in entry['requirements']
+        assert 'packages_required' in entry['requirements']
+        assert 'packages_recommended' in entry['requirements']
+        assert isinstance(entry['requirements']['details'], six.string_types)
+        assert isinstance(entry['requirements']['packages_required'], list)
+        assert isinstance(entry['requirements']['packages_recommended'], list)
 
     # Reset our matrix
     __reset_matrix()
@@ -1124,7 +1485,8 @@ def test_apprise_details_plugin_verification():
 
         # A Service Name MUST be defined
         assert 'service_name' in entry
-        assert isinstance(entry['service_name'], six.string_types)
+        assert isinstance(
+            entry['service_name'], (six.string_types, LazyTranslation))
 
         # Acquire our protocols
         protocols = parse_list(
@@ -1359,20 +1721,20 @@ def test_apprise_details_plugin_verification():
             if arg not in function_args:
                 # This print statement just makes the error easier to
                 # troubleshoot
-                print('{}:// template/arg/func reference missing error.'
-                      .format(protocols[0]))
-            assert arg in function_args
+                raise AssertionError(
+                    '{}.__init__() expects a {}=None entry according to '
+                    'template configuration'
+                    .format(SCHEMA_MAP[protocols[0]].__name__, arg))
 
         # Iterate over all of the function arguments and make sure that
         # it maps back to a key
         function_args -= valid_kwargs
         for arg in function_args:
             if arg not in map_to_entries:
-                # This print statement just makes the error easier to
-                # troubleshoot
-                print('{}:// template/func/arg reference missing error.'
-                      .format(protocols[0]))
-            assert arg in map_to_entries
+                raise AssertionError(
+                    '{}.__init__({}) found but not defined in the '
+                    'template configuration'
+                    .format(SCHEMA_MAP[protocols[0]].__name__, arg))
 
         # Iterate over our map_to_aliases and make sure they were defined in
         # either the as a token or arg
@@ -1414,7 +1776,7 @@ def test_apprise_details_plugin_verification():
 
 @pytest.mark.skipif(sys.version_info.major <= 2, reason="Requires Python 3.x+")
 @mock.patch('requests.post')
-@mock.patch('apprise.py3compat.asyncio.notify')
+@mock.patch('apprise.py3compat.asyncio.notify', wraps=py3aio.notify)
 def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
     """
     API: Apprise() async_mode tests
@@ -1474,8 +1836,8 @@ def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
 
     # Send Notifications Syncronously
     assert a.notify("sync") is True
-    # Verify our async code never got called
-    assert mock_async_notify.call_count == 0
+    # Verify our async code got called
+    assert mock_async_notify.call_count == 1
     mock_async_notify.reset_mock()
 
     # another way of looking a our false set asset configuration
